@@ -25,22 +25,36 @@ class RSA(object):
 		# the key's passphrase (Leave None for no passphrase).
 		passphrase=None,
 	):
-		self.directory = directory
+
+		# defaults.
 		self.passphrase = passphrase
-		self.public_key = public_key
-		self.private_key = private_key
-		self.id = self.directory
-		if self.id == None:
+		
+		# options.
+		if directory != None:
+			self.id = self.directory
+			self.directory = directory
+			self.private_key = directory+"/private_key.pem"
+			self.public_key = directory+"/public_key.pem"
+		else:
+			directory = Formats.FilePath(private_key).base()
+			self.private_key = private_key
+			self.public_key = public_key
 			self.id = str(self.public_key)
+			
 	# key management.
 	def generate_keys(self):
 
 		# default response.
 		response = utils.__default_response__()
 
+		# set dir,
+		directory = self.directory
+		private_key_path = self.private_key
+		public_key_path = self.public_key
+
 		# checks.
-		if self.directory != None and os.path.exists(self.directory): 
-			response["error"] = f"Key [{self.id}] already exists."
+		if os.path.exists(directory): 
+			response["error"] = f"Key [{directory}] already exists."
 			return response
 			
 		# generate.
@@ -54,34 +68,29 @@ class RSA(object):
 			private_key_pem = key_pair.exportKey(passphrase=self.passphrase)
 
 		# save.
-		if self.directory != None: 
-			os.mkdir(self.directory)
-			utils.__save_file__(self.directory+"/private_key.pem", private_key_pem.decode())
-			utils.__save_file__(self.directory+"/public_key.pem", public_key_pem.decode())
-			utils.__set_file_path_permission__(self.directory, permission=700)
-			utils.__set_file_path_permission__(self.directory+"/private_key.pem", permission=600)
-			utils.__set_file_path_permission__(self.directory+"/public_key.pem", permission=644)
+		os.mkdir(directory)
+		utils.__save_file__(private_key_path, private_key_pem.decode())
+		utils.__save_file__(public_key_path, public_key_pem.decode())
+		utils.__set_file_path_permission__(directory, permission=700)
+		utils.__set_file_path_permission__(private_key_path, permission=600)
+		utils.__set_file_path_permission__(public_key_path, permission=644)
 		
 		# response.
-		response["success"] = True
-		response["message"] = "Successfully generated a key pair."
-		return response
+		return r3sponse.success_response(f"Successfully generated a key {directory}.")
 
 		#
 	def load_keys(self):
 
 		# load keys.
 		response = utils.__default_response__()
-		if self.public_key == None:
-			self.public_key = utils.__load_file__(self.directory+"/public_key.pem")
-		if self.private_key == None:	
-			self.private_key = utils.__load_file__(self.directory+"/private_key.pem")
+		self.private_key_data = utils.__load_file__(self.private_key)
+		self.public_key_data = utils.__load_file__(self.public_key)
 
 		# initialize keys.
 		if self.passphrase == None:
 			try:
-				self.public_key_object = _RSA_.importKey(self.public_key)
-				self.private_key_object = _RSA_.importKey(self.private_key)
+				self.public_key_object = _RSA_.importKey(self.public_key_data)
+				self.private_key_object = _RSA_.importKey(self.private_key_data)
 			except ValueError as e:
 				if "Padding is incorrect" in str(e):
 					response["error"] = "Provided an incorrect passphrase."
@@ -89,8 +98,8 @@ class RSA(object):
 				return response
 		else:
 			try:
-				self.public_key_object = _RSA_.importKey(self.public_key, passphrase=self.passphrase)
-				self.private_key_object = _RSA_.importKey(self.private_key, passphrase=self.passphrase)
+				self.public_key_object = _RSA_.importKey(self.public_key_data, passphrase=self.passphrase)
+				self.private_key_object = _RSA_.importKey(self.private_key_data, passphrase=self.passphrase)
 			except ValueError as e:
 				if "Padding is incorrect" in str(e):
 					response["error"] = "Provided an incorrect passphrase."
@@ -98,22 +107,19 @@ class RSA(object):
 				return response
 		
 		# response.
-		response["success"] = True
-		response["message"] = "Successfully loaded the key pair."
-		return response
+		return r3sponse.success_response("Successfully loaded the key pair.")
 
 		#
 	def load_public_key(self):
 
 		# load keys.
 		response = utils.__default_response__()
-		if self.public_key == None:
-			self.public_key = utils.__load_file__(self.directory+"/public_key.pem")
+		self.public_key_data = utils.__load_file__(self.public_key)
 
 		# initialize keys.
 		if self.passphrase == None:
 			try:
-				self.public_key_object = _RSA_.importKey(self.public_key)
+				self.public_key_object = _RSA_.importKey(self.public_key_data)
 			except ValueError as e:
 				if "Padding is incorrect" in str(e):
 					response["error"] = "Provided an incorrect passphrase."
@@ -121,7 +127,7 @@ class RSA(object):
 				return response
 		else:
 			try:
-				self.public_key_object = _RSA_.importKey(self.public_key, passphrase=self.passphrase)
+				self.public_key_object = _RSA_.importKey(self.public_key_data, passphrase=self.passphrase)
 			except ValueError as e:
 				if "Padding is incorrect" in str(e):
 					response["error"] = "Provided an incorrect passphrase."
@@ -129,22 +135,19 @@ class RSA(object):
 				return response
 		
 		# response.
-		response["success"] = True
-		response["message"] = "Successfully loaded the key pair."
-		return response
+		return r3sponse.success_response("Successfully loaded the key pair.")
 
 		#
 	def load_private_key(self):
 
 		# load keys.
 		response = utils.__default_response__()
-		if self.private_key == None:	
-			self.private_key = utils.__load_file__(self.directory+"/private_key.pem")
+		self.private_key_data = utils.__load_file__(self.private_key)
 
 		# initialize keys.
 		if self.passphrase == None:
 			try:
-				self.private_key_object = _RSA_.importKey(self.private_key)
+				self.private_key_object = _RSA_.importKey(self.private_key_data)
 			except ValueError as e:
 				if "Padding is incorrect" in str(e):
 					response["error"] = "Provided an incorrect passphrase."
@@ -152,7 +155,7 @@ class RSA(object):
 				return response
 		else:
 			try:
-				self.private_key_object = _RSA_.importKey(self.private_key, passphrase=self.passphrase)
+				self.private_key_object = _RSA_.importKey(self.private_key_data, passphrase=self.passphrase)
 			except ValueError as e:
 				if "Padding is incorrect" in str(e):
 					response["error"] = "Provided an incorrect passphrase."
@@ -160,9 +163,7 @@ class RSA(object):
 				return response
 		
 		# response.
-		response["success"] = True
-		response["message"] = "Successfully loaded the key pair."
-		return response
+		return r3sponse.success_response("Successfully loaded the key pair.")
 
 		#
 	def edit_passphrase(self, passphrase=None):
@@ -171,7 +172,7 @@ class RSA(object):
 
 		if passphrase == None:
 			passphrase = '""'
-		output = utils.__execute__(["ssh-keygen", "-p", "-P", f"{self.passphrase}", "-N", f"{passphrase}", "-f", f"{self.directory}/private_key.pem"])
+		output = utils.__execute__(["ssh-keygen", "-p", "-P", f"{self.passphrase}", "-N", f"{passphrase}", "-f", f"{self.private_key}"])
 
 		# response.
 		response = utils.__default_response__()
@@ -199,9 +200,7 @@ class RSA(object):
 		if layers-1 > 0:
 			return self.encrypt_file(path, layers=layers-1)
 		else:
-			response["success"] = True
-			response["message"] = f"Successfully encrypted file [{path}]."
-			return response
+			return r3sponse.success_response(f"Successfully encrypted file [{path}].")
 
 		#
 	def encrypt_directory(self, path, recursive=False, layers=1):
@@ -227,9 +226,7 @@ class RSA(object):
 			if layers-1 > 0:
 				return self.encrypt_directory(path, recursive=True, layers=layers-1)
 			else:
-				response["success"] = True
-				response["message"] = f"Successfully encrypted directory [{path}] (recursively)."
-				return response		
+				return r3sponse.success_response(f"Successfully encrypted directory [{path}] (recursively).")
 
 
 		# encrypt the directory and save to x.encrypted.zip file.
@@ -255,8 +252,6 @@ class RSA(object):
 			response = utils.__default_response__()
 			response = self.encrypt_file(zip.file_path.path, layers=layers)
 			if response["success"]:
-				response["success"] = True
-				response["message"] = f"Successfully encrypted directory [{path}]."
 				file_path.delete(forced=True)
 				return response
 			else:
@@ -265,7 +260,7 @@ class RSA(object):
 				return response
 
 		#
-	def encrypt_string(self, string, layers=1):
+	def encrypt_string(self, string, layers=1, decode=True):
 
 		# encrypt.
 		response = utils.__default_response__()
@@ -277,7 +272,10 @@ class RSA(object):
 		else:
 			response["success"] = True
 			response["message"] = f"Successfully encrypted the string."
-			response["encrypted"] = encrypted.decode()
+			if decode:
+				response["encrypted"] = encrypted.decode()
+			else:
+				response["encrypted"] = encrypted
 			return response
 
 		#
@@ -299,7 +297,7 @@ class RSA(object):
 			return response
 
 		#
-	def decrypt_string(self, string, layers=1):
+	def decrypt_string(self, string, layers=1, decode=True):
 		
 		# decrypt.
 		response = utils.__default_response__()
@@ -312,7 +310,10 @@ class RSA(object):
 		else:
 			response["success"] = True
 			response["message"] = f"Successfully encrypted the string."
-			response["decrypted"] = decrypted.decode()
+			if decode:
+				response["decrypted"] = decrypted.decode()
+			else:
+				response["decrypted"] = decrypted
 			return response
 
 		#
